% returns an object of SA_VectorArray containing a vector of positions (element) 
% for each segment of the log polar scale. 
% The [2x1] Vectors in the elements are positions inside the
% corresponding segment.
%
% the segments are sorted beginning with the right segment at the bottom. The
% segments in the outmost circle are first counterclockwise. Then the second and 
% so forth.
% 
% parameter: InnerRadius: the radius of the innermost circle
%            DeltaRadius: the delta with wchich the radius increasesfor each
%                         new sector circle
%            Segments: the number of segments in wich the circle should be
%                      divided
%            Sectors: the number of sectors in which each segment should be
%                     divided 
% 
% The center of the PolarScale is [-0.5, -0.5]. The center is a point at 
% the corners of 4 pixels. This is such that there is an even spreading
% at small radii.

function PolarScale = SA_getLogPolarScale(InnerRadius, DeltaRadius, Segments, Sectors);
if (~nargout)
    error('No output argument found.');
end
if (nargin < 3) 
    error('Too less input arguments.');
end
if ((Segments <= 0) | (Sectors <= 0))
    error('Neither Segments nor Sectors may be smaller or equal than zero.');
end
if (InnerRadius <=0) 
    error('The radius may not be smaller or equal to zero.');
end
if (DeltaRadius <=0)
    error('DeltaRadius may not be smaller or equal to zero.');
end


% prepare VectorArray
PolarScale = SA_VectorArray;
for (i=1:Segments*Sectors)
    PolarScale = insert(PolarScale,[[]]); % insert empty element
end

% precompute some constants
indexAngle = 360/Segments;    % indexAngle in ]0;360]
f1 = Sectors;
f2 = Sectors - 1;
Radius = f1*InnerRadius + (f2*(f2+1))*(DeltaRadius/2);

% generate PolarScale
center = [-0.5,-0.5]; % center at the corner of 4 pixel
for (x=-(Radius+1) : Radius)
    for (y=-(Radius+1) : Radius+1)
        pos = [x,y] - center;
        length = sqrt(pos(1)^2 + pos(2)^2);
        if (length <= Radius) % point is inside of the cirle with r=Radius
            
            % compute angle in degree
            vec = pos ./ length;
            angle = rad2deg(asin(vec(2)));
            if (vec(1)>0) % right halve
                if (angle < 0)
                    angle = 360+angle;
                end
            else % vec(1) < 0
                angle = 180 - angle;
            end
            
            % compute index
            angleFactor = ceil(angle/indexAngle); % range: [0;Segments]
            if (angleFactor == 0)                 % range: [1;Segments]
                angleFactor = 1;
            end
            
            % compute segment factor solving
            r = length;
            segmentFactor = 0;
            f1 = 1;
            f2 = 0;
            while (r > 0) 
                segmentFactor = segmentFactor + 1;
                p = f1*InnerRadius + f2*(f2+1)*(DeltaRadius/2);
                r = length - p;
                f1 = f1 + 1;
                f2 = f2 + 1;
            end
            
            % scale segment factor
            segmentFactor = segmentFactor - 1;
            segmentFactor = (Sectors - 1) - segmentFactor; % inverse order
                
            index = Segments*segmentFactor + angleFactor;
            % append position
            PolarScale = appendToElement(PolarScale,index,[x,y]);
        end
    end
end
    
    
    
    
    
    