% returns an object of SA_VectorArray containing an element for each
% segment of the polar scale. The [2x1] Vectors in the elements are
% positions inside the corresponding segment.
%
% the segments are sorted beginning with the right segment at the bottom. The
% outmost segments are first counterclockwise. Then the second and so forth.
% 
% parameter: Radius: the radius of the outmost circle
%            Segments: the number of segments in wich the circle should be
%                      divided
%            Sectors: the number of sectors in which each segment should be
%                     divided 
% 
% The center of the PolarScale is [-0.5, -0.5]. The center is a point at 
% the corners of 4 pixels. This is such that there is an even spreading
% at small radii.

function PolarScale = SA_getPolarScale(Radius, Segments, Sectors);
if (~nargout)
    error('No output argument found.');
end
if (nargin < 3) 
    error('Too less input arguments.');
end
if ((Segments <= 0) | (Sectors <= 0))
    error('Neither Segments nor Sectors may be smaller or equal than zero.');
end
if (Radius <=0) 
    error('The radius may not be smaller or equal to zero.');
end



% prepare VectorArray
PolarScale = SA_VectorArray;
for (i=1:Segments*Sectors)
    PolarScale = insert(PolarScale,[[]]); % insert empty element
end

% precompute some constants
indexAngle = 360/Segments;    % indexAngle in ]0;360] = [1; 360]
indexRadius = Radius/Sectors; % indexRadius in ]0;Radius] = [1;Radius]

% generate PolarScale
center = [-0.5,-0.5]; % center at the corner of 4 pixel
for (x=-(Radius+1) : Radius)
    for (y=-(Radius+1) : Radius+1)
        pos = [x,y] - center;
        length = sqrt(pos(1)^2 + pos(2)^2);
        if (length <= Radius) % point is inside of the cirle with r=Radius
            
            % compute angle in degree
            vec = pos ./ length;
            angle = rad2deg(asin(vec(2)));
            if (vec(1)>0) % right halve
                if (angle < 0)
                    angle = 360+angle;
                end
            else % vec(1) < 0
                angle = 180 - angle;
            end
            
            % compute index
            angleFactor = ceil(angle/indexAngle); % range: [0;Segments]
            if (angleFactor == 0)                 % range: [1;Segments]
                angleFactor = 1;
            end
            segmentFactor = ceil((Radius-length)/indexRadius); % range: [0;Sectors]
            if (segmentFactor == 0)                            % range: [1;Sectors]
                segmentFactor = 1;
            end
            segmentFactor = segmentFactor - 1;                 % range: [0;Sectors-1]
            
            index = Segments*segmentFactor + angleFactor;
            % append position
            PolarScale = appendToElement(PolarScale,index,[x,y]);
        end
    end
end
    
    
    
    
    
    