% This function computes the Waterman-Smith alignment score for two stings
% of vectors
%
% The parameter strings String1 and String2 are expected to have the
% following form:
% [N x k] Matrix with N the size of the string and k the lenght of the
% vector. Bothe String1 and String2 are expected to have the same k. N may
% be different.
%
% GapPenalty is expected to be a float number. It is used in the algorithm
% to penalise gaps
%
% The return parameter Score is a [(k+1) x 1] Matrix. The first element is
% the score of the alignment process itself. The k remaining elements
% represent the optimal alignment. 1 = match, 2 = gap in String2, 3 = gap
% in String1, -1 = ignore element in String1, -2 = ignore element in
% String2


function Score = SA_globalAlignment(String1, String2, GapPenalty);
if (nargin < 3) 
    error('Too less input parameter');
end
if (size(String1,2) ~= size(String2,2)) 
    error('The vectors of the string (row size of String1 and String2) dont have the same size.');
end

k = size(String1,2); % size of the vectors to be compared
M = size(String1,1); % size of the first string (height of sa-matrix)
N = size(String2,1); % size of the second string (width of sa_matrix)

% in the Smith-Waterman algorithm, we start with the first row and first
% column all zeros. As our matrix is initilized with zeros we can just
% run the algorithm for the second row and column.
sa_matrix = zeros(M+1,N+1); % +1 because first row and column are placeholder 
                            % with zeros 
                            
sa_align = ones(M+1, N+1)*4; % 4 = begin new alignment

%%%%%%%%%%%
%PARAMETER%
%%%%%%%%%%%

% gap penalty parameter:
d = GapPenalty;

% record max score
max_score = 0;
max_score_i = 0;
max_score_j = 0;

% here we go:
for (j=2:N+1) % for each column
    for (i=2:M+1) % each row element
        % vectors to compare:
        v1 = String1(i-1,:)';
        v2 = String2(j-1,:)';
        
        % options for the path:
        F1 = sa_matrix(i-1,j-1) + score(v1,v2); % match = arrow diagonal up-left
        F2 = sa_matrix(i-1,j) - d; % gap in String2 = arrow up
        F3 = sa_matrix(i,j-1) - d; % gap in String1 = arrow left
        F4 = 0;
        
        [F,I] = max([F1,F2,F3,F4]);
        
        % update matrix
        sa_matrix(i,j) = F;
        sa_align(i,j) = I;
        
        if (F > max_score)
            max_score = F;
            max_score_i = i;
            max_score_j = j;
        end
    end
end

Score = max_score;
%sa_matrix
%sa_align

% compute alignment backtracking
%m = max_score_i; 
%n = max_score_j;
%alignment = [];
%while ((m>0) & (n>0) & (sa_align(m,n)~=4))
%    alignment = [sa_align(m, n); alignment];
%    if (sa_align(m,n) == 1) % diagonal
%        m = m-1; 
%        n = n-1;
%    elseif (sa_align(m,n) == 2) % up
%        m = m-1;
%    else  %(sa_align(m,n) == 3) % left
%        n = n-1;
%    end
%end

m = max_score_i; 
n = max_score_j;
alignlen = 0;
while ((m>0) & (n>0) & (sa_align(m,n)~=4))
    alignlen = alignlen + 1;
    if (sa_align(m,n) == 1) % diagonal
        m = m-1; 
        n = n-1;
        
    elseif (sa_align(m,n) == 2) % up
        m = m-1;
    else  %(sa_align(m,n) == 3) % left
        n = n-1;
    end
end

Score = [Score; alignlen];




% the function score is the used similarity function for the alignment process
% here it is the sample correlation of the two vectors x and y.
% x and y are expected to have the same size and to be [k x 1] vectors.
function Value = score(x, y);
if ((size(x,1) ~= size(y,1)) | (size(x,2) ~= size(y,2)))
    error('In function F: the parameter vectors x and y have different size');
end

n = size(x,1);

% compute the mean values of x and y
sum_x = 0;
sum_y = 0;
for (i=1:n) 
    sum_x = sum_x + x(i);
    sum_y = sum_y + y(i);
end
x_mean = sum_x / n;
y_mean = sum_y / n;

% compute sum((x - x_mean)*(y - y_mean))
% and
% compute standard deviation s_x and s_y for the vectors x and y
sum = 0;
s_x = 0;
s_y = 0;
for (i=1:n)
    sum = sum + ((x(i) - x_mean)*(y(i) - y_mean));
    
    s_x = s_x + (x(i) - x_mean)^2;  
    s_y = s_y + (y(i) - y_mean)^2;
end
s_x = sqrt(s_x/n) + 0.000001; % + epsilon to prevent division by zero
s_y = sqrt(s_y/n) + 0.000001; % + epsilon to prevent division by zero

% compute the correlation
Value = sum/((n - 1)*s_x*s_y);

